﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ADODB;
using System.DirectoryServices;
using Redemption;

namespace DoNotArchiveChecker
{
    class Program
    {

        public static bool debugmode = false;



        public static string g_ExchangeServer;
        public static string g_legacyExchangeDN;

        static void Main(string[] args)
        {
           // if (!CheckCommandLineValues(args))
           //     return;

            g_ExchangeServer = "EXCH1";

            if (debugmode)
                doutput("Debug Mode is turned on");


            // Get list of mailboxes

            string defaultNamingContext;
            DirectoryEntry rootDSE = new DirectoryEntry("LDAP://RootDSE");
            defaultNamingContext = rootDSE.Properties["defaultNamingContext"].Value.ToString();
            if (debugmode)
                doutput("Default Naming Context :" + defaultNamingContext);

            DirectoryEntry entry = new DirectoryEntry("LDAP://CN=Configuration," + defaultNamingContext);
            DirectorySearcher search = new DirectorySearcher(entry)
            {
                SearchScope = SearchScope.Subtree,
                Filter = "(&(objectCategory=msExchExchangeServer)(cn=" + g_ExchangeServer + "))"
            };

            search.PropertiesToLoad.Add("legacyExchangeDN");
            SearchResult result = search.FindOne();
            if (result != null)
            {
                if (debugmode)
                    doutput("  Found -> " + result.GetDirectoryEntry().Path.ToString());

                ResultPropertyCollection fields = result.Properties;
                foreach (String LDAPField in fields.PropertyNames)
                {
                    foreach (Object myC in fields[LDAPField])
                    {
                        if (debugmode)
                            doutput("     " + LDAPField + " : " + myC.ToString());

                        if (LDAPField == "legacyexchangedn")
                        {
                            g_legacyExchangeDN = myC.ToString();
                        }
                    }

                }
            }
            else
            {
                Console.WriteLine("Exchange server specified on command line could not be found: " + g_ExchangeServer);
                return;
            }

            if (debugmode)
                doutput("LegacyExchangeDN of the Exchange server is " + g_legacyExchangeDN);
            

            entry = null;
            search = null;

            entry = new DirectoryEntry("LDAP://" + defaultNamingContext);
            search = new DirectorySearcher(entry)
            {
                SearchScope = SearchScope.Subtree,
                PageSize = 100,
                Filter = "(&(&(&(& (mailnickname=*)(!msExchHideFromAddressLists=TRUE)(|(&(objectCategory=person)(objectClass=user)(msExchHomeServerName=" + g_legacyExchangeDN + ")) )))))"
            };

            search.PropertiesToLoad.Add("mailnickname");
            SearchResultCollection results = search.FindAll();
            if (debugmode)
                doutput("Number of results found: " + results.Count);

            doutput("Starting to process: " + string.Format("{0:HH:mm:ss tt}", DateTime.Now));
            if (result != null)
            {
                foreach (SearchResult r in results)
                {
                    foreach (String PropertyName in r.Properties.PropertyNames)
                    {
                        foreach (Object rEntry in r.Properties[PropertyName])
                        {
                            if (PropertyName == "mailnickname")
                            {
                               
                                doutput("  >> Processing << " + rEntry);
                            
                                //  Process this mailbox
                                ProcessMbx(g_ExchangeServer, rEntry.ToString());
                                if (debugmode)
                                    Console.ReadLine();
                            }
                        }
                    }
                }
                

            }
            else
            {
                Console.WriteLine("No users found on this server.");
                return;
            }

            doutput("Finished processing: " + string.Format("{0:HH:mm:ss tt}", DateTime.Now));  
    
            Console.WriteLine("Press return");
            Console.ReadLine();
          
        }

        public static void ProcessMbx(String sExchangeServerName, String sMbxNickName)
        {
            RDOSession session = new RDOSession();
            try
            {
                session.LogonExchangeMailbox(sMbxNickName, sExchangeServerName);
            }
            catch (Exception e)
            {
                Console.WriteLine("{0} Exception logged.", e);
                return;
            }

            RDOFolder inboxFolder = session.GetDefaultFolder(rdoDefaultFolders.olFolderInbox);
            RDOFolder TOISFolder = inboxFolder.Parent;
            
            // Process each folder
            CheckFolder(TOISFolder, session, 1);
            session.Logoff();
            session = null;

        }

        public static void CheckFolder(RDOFolder f, RDOSession s, int depth)
        {
            string oStr = "      ";
            for (int i = 1; i <= depth; i++)
            {
                oStr = oStr + " ";
            }

            Console.WriteLine(oStr + " " + f.Name);

            // Check message on current folder
            foreach (RDOMail Msg in f.HiddenItems)
            {
                if (Msg.MessageClass.ToString().Length >= 26 && Msg.MessageClass.ToUpper().Substring(0,26) == "IPM.ENTERPRISEVAULT.FILTER")
                {
                    var Utils = new MAPIUtils();
                    var PropList = Utils.HrGetPropList(Msg, true);

                    var objSafeItem = new SafeMailItem();
                    objSafeItem.Item = Msg;

                    for (int lcounter = 1; lcounter <= PropList.Count; lcounter++)
                    {
                        int lPropTag = PropList[lcounter];
                        var vProperty = objSafeItem.Fields[lPropTag];
                        MAPITags ptag = (MAPITags)PropList[lcounter];

                        if ((lPropTag & 0x80000000) == 0x80000000)
                        {
                            //  It is an EV attribute
                            var vID = Utils.GetNamesFromIDs(objSafeItem, lPropTag).ID;

                            if (vID.ToString() == "Filter ID")
                            {
                                if (vProperty.ToString() == "2")
                                    Console.WriteLine(oStr + " - Do Not Archive is set on above folder");
                            }
                        }
                    }

                }
            }

            //  Repeat for subfolder
            depth = depth + 1;
            foreach (RDOFolder sfolder in f.Folders)
            {
                CheckFolder(sfolder, s, depth);
            }
        }

        public static bool CheckCommandLineValues(string[] args)
        {
            if (args.Length < 1 || args.Length > 2)  // No parameters - we can not continue
            {
                doutput("\n\n*** Insufficient arguments, need mode (1 or 0), followed by name of Exchange Server ***\n\n");
                return false;
            }


            // Set debugmode based on first parameter
            int testValue = int.Parse(args[0].ToString());

            if (testValue > 0)
                debugmode = true;

            g_ExchangeServer = args[1].ToString();

            // Optional fourth parameter is the name of a folder
            return true;
        }

        public static void doutput(string t)
        {
            Console.WriteLine(t);
        }




    }
}
